import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:mobile_pos/mobile_pos_sdk.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/logger.dart';
import 'package:mobile_pos/src/models/bluetooth_model.dart';
import 'package:mobile_pos/src/models/raven_const.dart';
import 'package:mobile_pos/src/views/home/raven_pay_app.dart';
import 'mobile_pos_platform_interface.dart';
import 'src/helpers/helper_functions.dart';

/// An implementation of [MobilePosPlatform] that uses method channels.
class MethodChannelMobilePos extends MobilePosPlatform {
  /// The method channel used to interact with the native platform.
  static ValueNotifier bluetoothConnected = ValueNotifier(false);
  @visibleForTesting
  final methodChannel = const MethodChannel('mobile_pos');

  @override
  Future<String?> getPlatformVersion() async {
    final version =
        await methodChannel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  @override
  void initMethodChannel() {
    channelStreamController = StreamController<dynamic>.broadcast();
    methodChannel.setMethodCallHandler((call) async {
      if (call.method == RavenMPosConst.onAlert) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.alert,
            data: call.arguments['alert']));
        logData(call.arguments['alert']);
        return;
      }
      if (call.method == RavenMPosConst.onError) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.alert,
            data: call.arguments['error']));
        return;
      }
      if (call.method == RavenMPosConst.onBluetoothStartScanning) {
        channelStreamController!.add(
            RavenMPosEvent(eventType: RavenMPosEventType.scanning, data: true));
        return;
      }

      if (call.method == RavenMPosConst.onBluetoothStopScanning) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.stopScanning, data: true));
        return;
      }

      if (call.method == RavenMPosConst.onBluetoothDisconnected) {
        bluetoothConnected.value = false;
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.disconnected, data: true));
        return;
      }

      if (call.method == RavenMPosConst.onBluetoothDeviceDetected) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.searchedDevices,
            data: BluetoothModel.fromJson(jsonDecode(call.arguments))));
        return;
      }
      if (call.method == RavenMPosConst.onBluetoothConnected) {
        bluetoothConnected.value = true;
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.connected, data: true));
        return;
      }
      if (call.method == RavenMPosConst.onBluetoothConnectionError) {
        bluetoothConnected.value = false;
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.connectionErr, data: null));
        return;
      }
      if (call.method == RavenMPosConst.noDeviceDetectedAnymore) {
        bluetoothConnected.value = false;
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.deviceNotDetectedAnymore,
            data: null));
        return;
      }
      if (call.method == RavenMPosConst.onFinishedSettingBankBox) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.settingCompleted, data: null));
        return;
      }
      if (call.method == RavenMPosConst.onRequestCard) {
        channelStreamController!.add(
            RavenMPosEvent(eventType: RavenMPosEventType.card, data: null));
        return;
      }

      if (call.method == RavenMPosConst.onRequestPin) {
        channelStreamController!
            .add(RavenMPosEvent(eventType: RavenMPosEventType.pin, data: null));
        return;
      }
      if (call.method == RavenMPosConst.onProcessCardPayment) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.processPayment,
            data: call.arguments));
        return;
      }

      if (call.method == RavenMPosConst.onTransactionError) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.transactionError,
            data: call.arguments['error']));
        return;
      }
      if (call.method == RavenMPosConst.onSettingBankBox) {
        channelStreamController!.add(RavenMPosEvent(
            eventType: RavenMPosEventType.settingBankBox, data: null));
        return;
      }
    });
  }

  @override
  Future<void> startRavenPay(BuildContext context) async {
    initMethodChannel();
    await Navigator.push(
      context,
      PageRouteBuilder(
        settings: const RouteSettings(name: "raven_pay"),
        transitionDuration: const Duration(milliseconds: 350),
        pageBuilder: (_, __, ___) => const RavenPayApp(),
        transitionsBuilder: (_, animation, __, child) {
          return SlideTransition(
            position: Tween<Offset>(
              begin: const Offset(0, 1),
              end: Offset.zero,
            ).animate(animation),
            child: child,
          );
        },
      ),
    );

    return;
  }

  @override
  Future<String?> chargeCard(BuildContext context,
      {required double amount,
      required String accountType,
      String? pin,
      required ConnectivityType connectivityType}) async {
    var args = <String, dynamic>{
      "amount": amount,
      'account_type': '00',
      'device_type': _getConnectivityCode(connectivityType),
      //
      'terminal_id': keyDetails!.tid!,
      'port': keyDetails!.port!,
      'master_key': keyDetails!.clrmasterkey!,
      'pin_key': keyDetails!.clrpinkey!,
      'ip': keyDetails!.ip!,
      'session_key': keyDetails!.clrsesskey!,
      'serial_number': aflliateInfo!.serialNumber!,
      'mid': getMid(context).toString(),
      'business_name': getBusinessName(context),
    };

    if (pin != null) {
      args['pin'] = pin;
    }

    final res =
        await methodChannel.invokeMethod<String>('startTransaction', args);
    return res;
  }

  @override
  Future<void> closeRavenPay(BuildContext context) async {
    if (pluginConfig.onClose != null) {
      pluginConfig.onClose!.call();
    } else {
      Navigator.of(context, rootNavigator: true)
          .popUntil((route) => route.isFirst);
    }
  }

  int _getConnectivityCode(ConnectivityType connectivityType) {
    switch (connectivityType) {
      case ConnectivityType.bluetooth:
        return 0;
      case ConnectivityType.otg:
        return 0;
      default:
        return 00;
    }
  }

  @override
  void setConfig(BankBoxConfig config) {
    this.config = config;
  }

  @override
  Future<void> connectBluetooth(BluetoothModel model) async {
    final res = model.toJson();
    logData(res);
    methodChannel.invokeMethod(
        RavenMPosConst.onRequestBluetoothConnection, res);
  }

  @override
  Future<void> onPinRequest(String pin) async {
    final res = {'pin': pin};
    logData(res);
    methodChannel.invokeMethod(RavenMPosConst.onPINProvided, res);
  }

  @override
  Future<void> scanForMPosDevice() async {
    methodChannel.invokeMethod(RavenMPosConst.scanForMPosDevice);
    return;
  }

  @override
  Future<dynamic> initLocationStatus() async {
    return methodChannel.invokeMethod(RavenMPosConst.locationStatus);
  }

  @override
  Future<void> disconnectDevice() async {
    methodChannel.invokeMethod(RavenMPosConst.disconnectDevice);
    return;
  }

  @override
  Future<void> disconnectBluetoothDevice() async {
    return methodChannel.invokeMethod(RavenMPosConst.disconnectBluetoothDevice);
  }

  @override
  Future<void> cancelOnPinEntry() async {
    methodChannel.invokeMethod(RavenMPosConst.cancelPinEntry);
    return;
  }
}
